//
// Copyright (c) 2002
// Ronald Kevin Burton
//
// This material is provided "as is", with absolutely no warranty expressed
// or implied. This software is provided solely as part of ".NET Common Language
// Runtime Unleashed" to help illustrate concepts and principles described in 
// that book. Any use is at your own risk.
//
// Permission to use or copy this software for any purpose is hereby granted 
// without fee, provided the above notices are retained on all copies.
// Permission to modify the code and to distribute modified code is granted,
// provided the above notices are retained, and a notice that the code was
// modified is included with the above copyright notice.
//
// 
using System;
using System.Reflection;

public class ReflectionTest
{
    private int fielda;
    private float fieldb;
    private string fieldc;
    ReflectionTest()
    {
       fielda = 0;
       fieldb = 0;
       fieldc = "";
    }
    ReflectionTest(int a, float b, String c)
    {
        fielda = a;
        fieldb = b;
        fieldc = c;
        return;
    }
    int getfielda()
    {
        return fielda;
    }
    float getfieldb()
    {
        return fieldb;
    }
    string getfieldc()
    {
        return fieldc;
    }
    int add(int a)
    {
        return a + fielda;
    }
    float add(float b)
    {
        return b + fieldb;
    }
    string add(string c)
    {
        return c + fieldc;
    }
    void Reflect()
    {
        try
        {
			Type t = GetType();
			Console.WriteLine("> " + t.BaseType.Name + " methods");
            MethodInfo [] methods = t.BaseType.GetMethods(BindingFlags.NonPublic | BindingFlags.Public | BindingFlags.Instance);
            for(int i = 0; i < methods.Length; i++)
            {
				Console.Write("   " + methods[i].Name);
				ParameterInfo [] p = methods[i].GetParameters();
				Console.Write("(");
				for(int j = 0; j < p.Length; j++)
					if(j != p.Length - 1)
						Console.Write(p[j].ParameterType + ", ");
					else
						Console.Write(p[j].ParameterType);
				Console.WriteLine(") ");
			}
            Console.WriteLine("> " + t.Name + " methods");
			methods = t.GetMethods(BindingFlags.NonPublic | BindingFlags.Public | BindingFlags.Instance);
			for(int i = 0; i < methods.Length; i++)
            {
				Console.Write("   " + methods[i].Name);
				ParameterInfo [] p = methods[i].GetParameters();
				Console.Write("(");
				for(int j = 0; j < p.Length; j++)
					if(j != p.Length - 1)
						Console.Write(p[j].ParameterType + ", ");
					else
						Console.Write(p[j].ParameterType);
				Console.WriteLine(") ");
			}
            Console.WriteLine("> " + t.BaseType.Name + " constructors");
            ConstructorInfo [] constructors = t.BaseType.GetConstructors(BindingFlags.NonPublic | BindingFlags.Public | BindingFlags.Instance);
            foreach(ConstructorInfo ci in constructors)
            {
                Console.Write("   " + ci.Name);
                Console.Write("(");
                foreach(ParameterInfo p in ci.GetParameters())
                    Console.Write(p.ParameterType + " ");
                Console.WriteLine(") ");
            }
            Console.WriteLine("> " + t.Name + " constructors");
            constructors = t.GetConstructors(BindingFlags.NonPublic | BindingFlags.Public | BindingFlags.Instance);
			foreach(ConstructorInfo ci in constructors)
			{
				Console.Write("   " + ci.Name);
				Console.Write("(");
				foreach(ParameterInfo p in ci.GetParameters())
					Console.Write(p.ParameterType + " ");
				Console.WriteLine(") ");
			}
			Console.WriteLine("> " + t.BaseType.Name + " fields");
            FieldInfo [] fields = t.BaseType.GetFields(BindingFlags.NonPublic | BindingFlags.Public | BindingFlags.Instance);
            for(int i = 0; i < fields.Length; i++)
            {
                Console.WriteLine("   " + fields[i].FieldType + " " + fields[i].Name);
            }
            Console.WriteLine("> " + t.Name + " fields");
            fields = t.GetFields(BindingFlags.NonPublic | BindingFlags.Public | BindingFlags.Instance);
            for(int i = 0; i < fields.Length; i++)
            {
                Console.WriteLine("   " + fields[i].FieldType + " " + fields[i].Name);
            }
		}
        catch(Exception e)
        {
            Console.WriteLine(e);
        }
    }
    public static void Main(String[] args)
    {
        ReflectionTest r = new ReflectionTest(1, 2F, "This is a test");
        r.Reflect();
    }
}
